""" version which can be consumed from within the module """
VERSION_STR = "0.0.38"
DESCRIPTION = "pymakehelper eases writing recipes when using the make system"
APP_NAME = "pymakehelper"
LOGGER_NAME = "pymakehelper"
